@@ -27,7 +27,7 @@ import os
 import sys
 import re
 
-from urlparse import *
+from urlparse import urlunsplit
 
 
 try:
@@ -45,7 +45,7 @@ if not os.path.isfile (os.path.join ('notify', 'all.py')):
 print 'Building extension...'
 
 # FIXME: Is that portable enough?
-if os.system ('./setup.py build_ext') != 0:
+if os.system ('python setup.py build_ext') != 0:
     sys.exit (1)
 
 
@@ -60,6 +60,7 @@ if fast_mode:
 
 sys.argv.extend (['--name=Py-notify',
                   '--url=http://home.gna.org/py-notify/',
+                  '--exclude=notify._2_5',
                   '--inheritance=grouped',
                   '--no-sourcecode',
                   '--css=%s'    % os.path.join ('docs', 'epydoc.css'),
