@@ -289,12 +289,37 @@ class AbstractStateTrackingCondition (AbstractCondition):
 
 
     def get (self):
+        """
+        Get the current state of the condition.  Since this condition type stores its
+        state, this is the last value as passed to C{L{_set}} method internally, except it
+        is always converted to either C{True} or C{False} using C{bool} function.
+
+        @rtype: C{bool}
+        """
+
         return self.__state
 
     def _set (self, value):
+        """
+        Set the state of the condition internally.  The C{value} parameter is first
+        coerced to boolean state using C{bool} function, then checked equality with the
+        current state.  So, this method does all that is needed for C{set} method of a
+        mutable condition.
+
+        This method I{must not} be used from outside.  For mutable conditions, use C{set}
+        instead; immutable ones update their states through other means.
+
+        @param  value: new value for the variable (coerced to boolean state
+                       automatically.)
+        @type   value: C{object}
+
+        @rtype:        C{bool}
+        @returns:      Whether condition state changed as a result.
+        """
+
         state = bool (value)
 
-        if self.get () is not state:
+        if self.get () != state:
             self.__state = state
             return self._value_changed (state)
         else:
@@ -304,7 +329,7 @@ class AbstractStateTrackingCondition (AbstractCondition):
     def _generate_derived_type_dictionary (cls, options):
         for attribute in (super (AbstractStateTrackingCondition, cls)
                           ._generate_derived_type_dictionary (options)):
-            if attribute[0] != 'get':
+            if attribute[0] not in ('get', 'set'):
                 yield attribute
 
         functions = {}
@@ -326,15 +351,39 @@ class AbstractStateTrackingCondition (AbstractCondition):
                   % AbstractValueObject._get_object (options)) in options, functions
 
         else:
+            if 'setter' in options:
+                setter_statement = ('setter (%s, initial_state)'
+                                    % AbstractValueObject._get_object (options))
+            else:
+                setter_statement = ''
+
             if object is not None:
                 exec (('def __init__(self, %s, initial_state):\n'
                        '    cls.__init__(self, initial_state)\n'
-                       '    %s = %s')
-                      % (object, AbstractValueObject._get_object (options), object)) \
+                       '    %s = %s\n'
+                       '    %s\n')
+                      % (object, AbstractValueObject._get_object (options), object,
+                         setter_statement)) \
                       in options, functions
             else:
-                exec ('def __init__(self, initial_state):\n'
-                      '    cls.__init__(self, initial_state)\n') in options, functions
+                exec (('def __init__(self, initial_state):\n'
+                       '    cls.__init__(self, initial_state)\n'
+                       '    %s\n')
+                      % setter_statement) \
+                      in options, functions
+
+        if 'setter' in options:
+            exec (('def _set (self, value):\n'
+                   '    state = bool (value)\n'
+                   '    if self.get () != state:\n'
+                   '        setter (%s, state)\n'
+                   '        self._AbstractStateTrackingCondition__state = state\n'
+                   '        return self._value_changed (state)\n'
+                   '    else:\n'
+                   '        return False')
+                  % AbstractValueObject._get_object (options)) in options, functions
+
+            exec 'def set (self, value): return self._set (value)' in options, functions
 
         for function in functions.iteritems ():
             yield function
@@ -370,6 +419,9 @@ class Condition (AbstractStateTrackingCondition):
         @param value: new I{state} of the condition (parameter name is just kept the same
                       as in the base class.)
         @type  value: C{object}
+
+        @rtype:       C{bool}
+        @returns:     Whether condition state changed as a result.
         """
 
         return self._set (value)
@@ -442,6 +494,12 @@ class PredicateCondition (AbstractStateTrackingCondition):
                 + super (PredicateCondition, self)._additional_description (formatter))
 
 
+    def _generate_derived_type_dictionary (cls, options):
+        raise TypeError ("`PredicateCondition' doesn't support derive_type() method")
+
+    _generate_derived_type_dictionary = classmethod (_generate_derived_type_dictionary)
+
+
 
 class WatcherCondition (AbstractStateTrackingCondition):
 
@@ -498,6 +556,9 @@ class WatcherCondition (AbstractStateTrackingCondition):
         @param  condition_to_watch: new condition to watch (copy state.)
         @type   condition_to_watch: C{L{AbstractCondition}} or C{None}
 
+        @rtype:                     C{bool}
+        @returns:                   Whether self state has changed as a result.
+
         @raises TypeError:          if C{condition_to_watch} is not an instance of
                                     C{L{AbstractCondition}} and not C{None}.
         @raises ValueError:         if C{condition_to_watch} is this condition.
@@ -511,6 +572,12 @@ class WatcherCondition (AbstractStateTrackingCondition):
                 raise TypeError ('can only watch other conditions')
 
         watched_condition = self.__get_watched_condition ()
+
+        if watched_condition is condition_to_watch:
+            return False
+
+        old_state = self.get ()
+
         if watched_condition is not None:
             watched_condition.changed.disconnect (self._set)
 
@@ -527,6 +594,8 @@ class WatcherCondition (AbstractStateTrackingCondition):
             elif watched_condition is not None and condition_to_watch is None:
                 AbstractGCProtector.default.unprotect (self)
 
+        return self.get () != old_state
+
 
     def __get_watched_condition (self):
         if self.__watched_condition is not None:
@@ -577,6 +646,12 @@ class WatcherCondition (AbstractStateTrackingCondition):
                 + super (WatcherCondition, self)._additional_description (formatter))
 
 
+    def _generate_derived_type_dictionary (cls, options):
+        raise TypeError ("`PredicateCondition' doesn't support derive_type() method")
+
+    _generate_derived_type_dictionary = classmethod (_generate_derived_type_dictionary)
+
+
 
 #-- Internal conditions ----------------------------------------------
 
@@ -920,10 +995,9 @@ class _IfElse (AbstractCondition):
 
     def __on_if_term_change (self, new_state):
         self.__term_state ^= 4
-        state              = _IfElse.__TERM_STATE_TO_SELF_STATE[self.__term_state]
 
-        if state != _IfElse.__TERM_STATE_TO_SELF_STATE[self.__term_state ^ 4]:
-            self._value_changed (state)
+        if self.__term_state & 3 == 1 or self.__term_state & 3 == 2:
+            self._value_changed (self.__term_state == 1 or self.__term_state == 6)
 
 
     def __on_then_term_change (self, new_state):
