@@ -112,6 +112,12 @@ from notify.bind  import Binding, WeakBinding
 from notify.gc    import AbstractGCProtector
 from notify.utils import raise_not_implemented_exception
 
+try:
+    import contextlib
+except ImportError:
+    # Ignore, related features will not be provided.
+    pass
+
 
 
 #-- Signal interface classes -----------------------------------------
@@ -121,22 +127,32 @@ class AbstractSignal (object):
     """
     Abstract interface all signal classes must implement.
 
-    Methods of this interface can be roughly grouped into the following groups:
+    @group Connecting Handlers:
+    is_connected, connect, connect_safe, do_connect, do_connect_safe, disconnect,
+    disconnect_all, connecting, connecting_safely
 
-      - Query methods: C{L{has_handlers}} (or just C{L{__nonzero__}}) and
-        C{L{count_handlers}}.
+    @group Blocking Handlers:
+    is_blocked, block, unblock, blocking
 
-      - Adding and removing handlers: C{L{is_connected}}, C{L{connect}},
-        C{L{connect_safe}}, C{L{do_connect}}, C{L{do_connect_safe}}, C{L{disconnect}} and
-        C{L{disconnect_all}}
+    @group Emission:
+    __call__, emit, stop_emission, emission_level, emission_stopped
 
-      - Blocking connected handlers from being invoked: C{L{is_blocked}}, C{L{block}} and
-        C{L{unblock}}.
+    @group Handler List Maintenance:
+    has_handlers, __nonzero__, count_handlers, collect_garbage
 
-      - Emission: C{L{emit}} (or just C{L{__call__}}), C{L{emission_level}},
-        C{L{emission_stopped}} and C{L{stop_emission}}.
+    @group Methods for Subclasses:
+    _wrap_handler, _additional_description
 
-      - Rarely needed: C{L{_wrap_handler}} and C{L{collect_garbage}}.
+    @group Internals:
+    _get_emission_level, _is_emission_stopped, __to_string
+
+    @sort:
+    is_connected, connect, connect_safe, do_connect, do_connect_safe, disconnect,
+    disconnect_all, connecting, connecting_safely,
+    is_blocked, block, unblock, blocking,
+    __call__, emit, stop_emission, emission_level, emission_stopped,
+    has_handlers, __nonzero__, count_handlers, collect_garbage,
+    _wrap_handler, _additional_description
     """
 
     __slots__ = ()
@@ -149,7 +165,7 @@ class AbstractSignal (object):
         values, post-process values after all handlers run (or emission stops for some
         reason) or stop emission based on the values.
 
-        Note that accumulator should I{not} contain the accumulated value or have any
+        Note that accumulator must I{not} contain the accumulated value or have any
         internal state (except that specified by C{__init__} arguments) whatsoever.  This
         no-OOP design is required to make accumulators thread- and reentrance-safe, so
         that the same accumulator can be used from multiple threads or nested signal
@@ -555,6 +571,24 @@ class AbstractSignal (object):
         raise_not_implemented_exception (self)
 
 
+    if 'contextlib' in globals ():
+        # This is a collection of gross hacks aimed at making this work (obviously) _and_
+        # tricking Epydoc into not noticing that we import stuff from a different module.
+
+        from notify._2_5 import signal as _2_5
+
+        connecting                   = _2_5.connecting
+        connecting_safely            = _2_5.connecting_safely
+        blocking                     = _2_5.blocking
+
+        # This is needed so that Epydoc sees docstrings as UTF-8 encoded.
+        connecting.__module__        = __module__
+        connecting_safely.__module__ = __module__
+        blocking.__module__          = __module__
+
+        del _2_5
+
+
     def emit (self, *arguments):
         """
         Invoke non-blocked handlers connected to C{self}, passing C{arguments} to them.
@@ -577,6 +611,9 @@ class AbstractSignal (object):
         """
         Same as C{L{emit}} method.
 
+        @note:
+        Don’t override this method, override C{emit} instead, if really needed.
+
         @rtype:   C{object}
         @returns: Value, determined by subclass and, possibly, by its
                   L{accumulator <AbstractAccumulator>}.
@@ -973,21 +1010,29 @@ class Signal (AbstractSignal):
 
 
     def disconnect (self, handler, *arguments):
-        if self._handlers is None or not callable (handler):
+        handlers = self._handlers
+        if handlers is None or not callable (handler):
             return False
 
         if arguments:
             handler = Binding (handler, arguments)
 
-        for index, _handler in enumerate (self._handlers):
-            if _handler == handler:
+        # Note: we must disconnect _last_ of equal connected handlers, in order to make
+        # connect()/disconnect() a no-op.  We use a custom loop because of that (and since
+        # reversed() only appeared in 2.4.)
+
+        index = len (handlers) - 1
+        while index >= 0:
+            if handlers[index] != handler:
+                index -= 1
+            else:
                 if self.__emission_level == 0:
-                    del self._handlers[index]
+                    del handlers[index]
                 else:
-                    self._handlers[index] = None
+                    handlers[index] = None
 
                 if (    self._blocked_handlers is not _EMPTY_TUPLE
-                    and handler not in self._handlers[index:]):
+                    and handler not in handlers[:index]):
                     # This is the last handler, need to make sure it is not listed in
                     # `_blocked_handlers'.
                     self._blocked_handlers = [_handler for _handler in self._blocked_handlers
@@ -996,7 +1041,7 @@ class Signal (AbstractSignal):
                     if not self._blocked_handlers:
                         self._blocked_handlers = _EMPTY_TUPLE
 
-                if not self._handlers:
+                if not handlers:
                     self._handlers = None
 
                 return True
@@ -1170,6 +1215,8 @@ class Signal (AbstractSignal):
 
 
 
+#-- Handler auto-disconnecting signal class --------------------------
+
 class CleanSignal (Signal):
 
     """
@@ -1282,6 +1329,8 @@ class CleanSignal (Signal):
 
 
 
+#-- Internal variables -----------------------------------------------
+
 # It is not guaranteed to be a singleton, although it probably always is.
 _EMPTY_TUPLE = ()
 
