@@ -36,7 +36,7 @@ if not os.path.isfile (os.path.join ('notify', 'all.py')):
 print 'Building extension...'
 
 # FIXME: Is that portable enough?
-if os.system ('./setup.py build_ext') != 0:
+if os.system ('python setup.py build_ext') != 0:
     sys.exit (1)
 
 
@@ -46,7 +46,7 @@ class AllTests (object):
     def __init__(self):
         everything = unittest.TestSuite ()
 
-        for module_name in ('all', 'bind', 'condition', '_gc', 'mediator', 'signal',
+        for module_name in ('all', 'base', 'bind', 'condition', '_gc', 'mediator', 'signal',
                             'utils', 'variable'):
             module = __import__('test.%s' % module_name, globals (), locals (), ('*',))
 
