@@ -31,8 +31,8 @@ if __name__ == '__main__':
 
 import unittest
 
-from notify.signal import *
-from test.__common import *
+from notify.signal import AbstractSignal, Signal
+from test.__common import NotifyTestCase
 
 
 
@@ -122,6 +122,30 @@ class SimpleSignalTestCase (NotifyTestCase):
         self.assert_results (1, 1, 1, 2, 2)
 
 
+    def test_connect_disconnect (self):
+        signal       = Signal ()
+        self.results = []
+
+        signal.connect (self.simple_handler)
+        signal.connect (self.simple_handler_100)
+
+        signal.emit (1)
+
+        # This must be a no-op.
+        signal.connect    (self.simple_handler)
+        signal.disconnect (self.simple_handler)
+
+        signal.emit (2)
+
+        # This must be a no-op.
+        signal.connect    (self.simple_handler_100)
+        signal.disconnect (self.simple_handler_100)
+
+        signal.emit (3)
+
+        self.assert_results (1, 101, 2, 102, 3, 103)
+
+
     def test_block (self):
         signal       = Signal ()
         self.results = []
@@ -399,14 +423,6 @@ class RecursiveEmissionSignalTestCase (NotifyTestCase):
         self.assert_results (0, 1, 2, 3, 104, 4, 104, 103, 102, 101, 100)
 
 
-    def simple_handler_100 (self, *arguments):
-        self.simple_handler (100 + arguments[0])
-
-
-    def simple_handler_200 (self, *arguments):
-        self.simple_handler (200 + arguments[0])
-
-
     def recursive_handler (self, *arguments):
         self.simple_handler (*arguments)
 
@@ -640,6 +656,13 @@ class ExoticSignalTestCase (NotifyTestCase):
 
 
 
+import __future__
+
+if NotifyTestCase.note_skipped_tests ('with_statement' in __future__.all_feature_names):
+    from test._2_5.signal import SignalContextManagerTestCase
+
+
+
 if __name__ == '__main__':
     unittest.main ()
 
